#!/usr/bin/env python
# -*- coding: utf-8 -*-

from distutils.core import setup

setup(name='tabellarius',
      version='0.0.2',
      description='A mail-sorting tool that is less annoying',
      author='Arnold Bechtoldt',
      author_email='mail@arnoldbechtoldt.com',
      url='https://github.com/bechtoldt/tabellarius',
      packages=['tabellarius'], )
